//***************************************************************************
// Group Linslot / Linux - Slotrace Manager
// File setup.cc
// Date 16.12.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//***************************************************************************

//***************************************************************************
// Include
//***************************************************************************

#include <QMessageBox>
#include <QHeaderView>

#include <common.hpp>
#include <setup.hpp>
#include <delegateitems.hpp>

//***************************************************************************
// Object
//***************************************************************************

SetupDialog::SetupDialog(QString configPath)
{ 
   int count;
   char tmpDevice[100];

   setupUi(this);

   QCoreApplication::setOrganizationName("Jrg Wendel");
   QCoreApplication::setApplicationName("linslot") ;

   settings = new QSettings(configPath + "/linslotrc", QSettings::IniFormat);
 
   settings->beginGroup("common");

   lapCount = settings->value("lapCount", 10).toInt();
   lapLimit = settings->value("lapLimit", 50).toInt();
   slotLength = settings->value("slotLength", 14.0).toDouble();
   maxTimeLapRace = settings->value("maxTimeLapRace", 0).toInt();
   maxTimeTraining = settings->value("maxTimeTraining", 0).toInt();
   speedFactor = settings->value("speedFactor", 1).toInt();
   abortAtJumpTheGun = settings->value("abortAtJumpTheGun", 0).toInt();
   strcpy(tmpDevice, settings->value("usbDevice", "/dev/usb/iowarrior").toString().toAscii());
   strcpy(courseName, settings->value("course", "Heimstrecke").toString().toAscii());
   strcpy(databaseName, settings->value("databaseName", "linslot.db").toString().toAscii());
   strcpy(resourcePath, settings->value("resourcePath", "/usr/local/share/linslot/").toString().toAscii());
   penaltyAtJumpTheGun = settings->value("penaltyAtJumpTheGun", 5).toInt();
   fuelMax = settings->value("fuelMax", 75.0).toDouble();
   fuelPerLap = settings->value("fuelPerLap", 9.5).toDouble();
   fuelingDelay = settings->value("fuelingDelay", 2).toInt();
   fuelingPerSecond = settings->value("fuelingPerSecond", 15).toInt();
   fuelFactorFastLap = settings->value("fuelFactorFastLap", 8).toInt();
   fuelFactorSlowLap = settings->value("fuelFactorSlowLap", 8).toInt();
   averageLap = settings->value("averageLap", 12.5).toDouble();
   fuelingActive = settings->value("fuelingActive", yes).toInt();
   fuelPenaltyTime = settings->value("fuelPenaltyTime", 5).toInt();

   settings->endGroup();

   settings->beginGroup("driver");
   count = settings->beginReadArray("driver");

   for (int i = 0; i < count; ++i) 
   {
      settings->setArrayIndex(i);
      listWidgetDriver->addItem(settings->value("name").toString());
   }

   settings->endArray();
   settings->endGroup();

   tabWidget->setCurrentIndex(0);

   // read io signal definition

   settings->beginGroup("inputSignals");
   count = settings->beginReadArray("inputSignals");

   if (count > bitInputCount)
   {
      tell(eloAlways, "Warning: To many input signals configured, ignoring");
      count = bitInputCount;
   }

   for (int i = 0; i < count; ++i) 
   {
      settings->setArrayIndex(i);

      inputBits[i].bit = settings->value("bit").toInt();
      inputBits[i].mode = (TriggerEdge)settings->value("mode").toInt();
   }

   settings->endArray();
   settings->endGroup();

   // output signals

   settings->beginGroup("outputSignals");
   count = settings->beginReadArray("outputSignals");

   if (count > bitOutputCount)
   {
      tell(eloAlways, "Warning: To many output signals configured, ignoring");
      count = bitOutputCount;
   }

   for (int i = 0; i < count; ++i) 
   {
      settings->setArrayIndex(i);

      outputBits[i].bit = settings->value("bit").toInt();
      outputBits[i].mode = (OutputMode)settings->value("mode").toInt();
   }

   settings->endArray();
   settings->endGroup();

   // todo other devices ...

   comboBoxDevice->addItem("/dev/ttyUSB0");
   comboBoxDevice->addItem("/dev/ttyUSB1");
   comboBoxDevice->addItem("/dev/ttyUSB2");
   comboBoxDevice->addItem("/dev/ttyUSB3");
   comboBoxDevice->addItem("/dev/usb/iowarrior");
   comboBoxDevice->setCurrentIndex(comboBoxDevice->findText(tmpDevice));

   lineEditCourse->setText(courseName);
   lineEditDbName->setText(databaseName);
   lineEditResourcePath->setText(resourcePath);
   doubleSpinBoxSlotLength->setValue(slotLength);
   spinBoxLapCount->setValue(lapCount);
   spinBoxTrainingLapLimit->setValue(lapLimit);
   spinBoxJumpTheGunPenaltyTime->setValue(penaltyAtJumpTheGun);

   doubleSpinBoxFuelPerLap->setValue(fuelPerLap);
   doubleSpinBoxFuelMax->setValue(fuelMax);
   spinBoxFuelingDelay->setValue(fuelingDelay);
   spinBoxFuelingPerSecond->setValue(fuelingPerSecond);
   spinBoxFuelFactorFastLap->setValue(fuelFactorFastLap);
   spinBoxFuelFactorSlowLap->setValue(fuelFactorSlowLap);
   doubleSpinBoxAverageLap->setValue(averageLap);
   checkBoxFueling->setChecked(fuelingActive);
   groupBoxFueling->setEnabled(fuelingActive);
   spinBoxFuelPenaltyTime->setValue(fuelPenaltyTime);

   if (abortAtJumpTheGun)
      radioButtonJumpTheGunAbort->setChecked(true);
   else
      radioButtonJumpTheGunPenalty->setChecked(true);
      
   switch (speedFactor)
   {
      case 1:  radioButtonFaktor1->setChecked(true);   break;
      case 24: radioButtonFaktor24->setChecked(true);  break;
      case 32: radioButtonFaktor32->setChecked(true);  break;
      case 43: radioButtonFaktor43->setChecked(true);  break;
      default: radioButtonFaktor1->setChecked(true);   break;
   }

   // fill io table views

   // inputs

   tableWidgetInputSignals->setRowCount(bitInputCount);
   tableWidgetInputSignals->setColumnCount(3);

   for (int i = 0; i < bitInputCount; ++i) 
   {
      QTableWidgetItem* itemFunction = new QTableWidgetItem(inputFunctionName(i));
      itemFunction->setFlags(itemFunction->flags() & ~Qt::ItemIsEditable);
      tableWidgetInputSignals->setItem(i, 0, itemFunction);

      QTableWidgetItem* itemBit = new QTableWidgetItem(inputBits[i].bit == na ? "NA" : QString::number(inputBits[i].bit));
      tableWidgetInputSignals->setItem(i, 1, itemBit);

      QTableWidgetItem* itemMode = new QTableWidgetItem(inputBits[i].mode == 0 ? "rising" : "falling");
      tableWidgetInputSignals->setItem(i, 2, itemMode);
   }

   QStringList labels;
   labels << tr("Funktion") << tr("Bit") << tr("Trigger-Flanke");

   tableWidgetInputSignals->setHorizontalHeaderLabels(labels);
   tableWidgetInputSignals->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
   tableWidgetInputSignals->horizontalHeader()->setResizeMode(1, QHeaderView::Fixed);
   tableWidgetInputSignals->horizontalHeader()->setResizeMode(2, QHeaderView::Fixed);
   tableWidgetInputSignals->verticalHeader()->hide();

   ComboBoxDelegate* modeDelegate = new ComboBoxDelegate(this);
   modeDelegate->addItem("rising");
   modeDelegate->addItem("falling");
   tableWidgetInputSignals->setItemDelegateForColumn(2, modeDelegate);

   ComboBoxDelegate* bitDelegate = new ComboBoxDelegate(this);
   bitDelegate->addItem("NA");
   for (int i = 0; i < 32; i++) 
      bitDelegate->addItem(QString::number(i));
   tableWidgetInputSignals->setItemDelegateForColumn(1, bitDelegate);

   connect(tableWidgetInputSignals, SIGNAL(cellChanged(int, int)), 
           this, SLOT(inputSignalChanged(int, int)));

   // outputs

   tableWidgetOutputSignals->setRowCount(bitOutputCount);
   tableWidgetOutputSignals->setColumnCount(3);

   for (int i = 0; i < bitOutputCount; ++i) 
   {
      QTableWidgetItem* itemFunction = new QTableWidgetItem(outputFunctionName(i));
      itemFunction->setFlags(itemFunction->flags() & ~Qt::ItemIsEditable);
      tableWidgetOutputSignals->setItem(i, 0, itemFunction);

      QTableWidgetItem* itemBit = new QTableWidgetItem(outputBits[i].bit == na ? "NA" : QString::number(outputBits[i].bit));
      tableWidgetOutputSignals->setItem(i, 1, itemBit);

      QTableWidgetItem* itemMode = new QTableWidgetItem(outputBits[i].mode == 0 ? "normal" : "pwm");
      tableWidgetOutputSignals->setItem(i, 2, itemMode);
   }

   QStringList olabels;
   olabels << tr("Funktion") << tr("Bit") << tr("Mode");

   tableWidgetOutputSignals->setHorizontalHeaderLabels(olabels);
   tableWidgetOutputSignals->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
   tableWidgetOutputSignals->horizontalHeader()->setResizeMode(1, QHeaderView::Fixed);
   tableWidgetOutputSignals->horizontalHeader()->setResizeMode(2, QHeaderView::Fixed);
   tableWidgetOutputSignals->verticalHeader()->hide();

   ComboBoxDelegate* omodeDelegate = new ComboBoxDelegate(this);
   omodeDelegate->addItem("normal");
   omodeDelegate->addItem("pwm");
   tableWidgetOutputSignals->setItemDelegateForColumn(2, omodeDelegate);

   ComboBoxDelegate* obitDelegate = new ComboBoxDelegate(this);
   obitDelegate->addItem("NA");
   for (int i = 0; i < 32; i++) 
      obitDelegate->addItem(QString::number(i));
   tableWidgetOutputSignals->setItemDelegateForColumn(1, obitDelegate);

   connect(tableWidgetOutputSignals, SIGNAL(cellChanged(int, int)), 
           this, SLOT(outputSignalChanged(int, int)));
}

SetupDialog::~SetupDialog()
{
   tell(eloAlways, "save settings");

   settings->beginGroup("common");

   settings->setValue("maxTimeTraining", maxTimeTraining);
   settings->setValue("maxTimeLapRace", maxTimeLapRace);
   settings->setValue("speedFactor", speedFactor);
   settings->setValue("slotLength", slotLength);
   settings->setValue("lapCount", lapCount);
   settings->setValue("lapLimit", lapLimit);
   settings->setValue("usbDevice", usbDevice);
   settings->setValue("course", courseName);
   settings->setValue("databaseName", databaseName);
   settings->setValue("resourcePath", resourcePath);
   settings->setValue("abortAtJumpTheGun", abortAtJumpTheGun);
   settings->setValue("penaltyAtJumpTheGun", penaltyAtJumpTheGun);
   settings->setValue("fuelMax", fuelMax);
   settings->setValue("fuelPerLap", fuelPerLap);
   settings->setValue("fuelingDelay", fuelingDelay);
   settings->setValue("fuelingPerSecond", fuelingPerSecond);
   settings->setValue("fuelFactorFastLap", fuelFactorFastLap);
   settings->setValue("fuelFactorSlowLap", fuelFactorSlowLap);
   settings->setValue("averageLap", averageLap);
   settings->setValue("fuelingActive", fuelingActive);
   settings->setValue("fuelPenaltyTime", fuelPenaltyTime);

   settings->endGroup();

   settings->beginGroup("driver");
   settings->beginWriteArray("driver");

   for (int i = 0; i < listWidgetDriver->count(); ++i) 
   {
      settings->setArrayIndex(i);
      settings->setValue("name", listWidgetDriver->item(i)->text());
   }

   settings->endArray();
   settings->endGroup();

   // input signals

   settings->beginGroup("inputSignals");
   settings->beginWriteArray("inputSignals");

   for (int i = 0; i < bitInputCount; ++i) 
   {
      settings->setArrayIndex(i);

      settings->setValue("bit", inputBits[i].bit);
      settings->setValue("mode", inputBits[i].mode);
   }

   settings->endArray();
   settings->endGroup();

   // output signals

   settings->beginGroup("outputSignals");
   settings->beginWriteArray("outputSignals");

   for (int i = 0; i < bitOutputCount; ++i) 
   {
      settings->setArrayIndex(i);

      settings->setValue("bit", outputBits[i].bit);
      settings->setValue("mode", outputBits[i].mode);
   }

   settings->endArray();
   settings->endGroup();

   delete settings;
}

//***************************************************************************
// On Button Add Driver
//***************************************************************************

void SetupDialog::on_pushButtonAddDriver_clicked()
{
   if (!lineEditDriver->text().isEmpty())
   {
      QListWidgetItem* listItem;

      for (int i = 0; i < listWidgetDriver->count(); i++)
      {
         if ((listItem = listWidgetDriver->item(i)))
            if (listItem->text().compare(lineEditDriver->text()) == 0)
            {
               QMessageBox::information(this, "Info", "Der Fahrer ist bereits in der Liste enthalten!");
               return ;
            }
      }

      listWidgetDriver->addItem(lineEditDriver->text());
      lineEditDriver->setText("");
      lineEditDriver->setFocus();
   }
}

//***************************************************************************
// On Button Del Driver
//***************************************************************************

void SetupDialog::on_pushButtonDelDriver_clicked()
{
   if (listWidgetDriver->currentRow() >= 0)
      delete listWidgetDriver->takeItem(listWidgetDriver->currentRow());
}

//***************************************************************************
// On Lap Limit Change
//***************************************************************************

void SetupDialog::on_spinBoxTrainingLapLimit_valueChanged(int value)
{
   lapLimit = value;
}

//***************************************************************************
// On Lap Count Change
//***************************************************************************

void SetupDialog::on_spinBoxLapCount_valueChanged(int value)
{
   lapCount = value;
}

//***************************************************************************
// On Slot Lenght Change
//***************************************************************************

void SetupDialog::on_doubleSpinBoxSlotLength_valueChanged(double value)
{
   slotLength = value;
}

//***************************************************************************
// On Usb Device Editing Finished
//***************************************************************************

void SetupDialog::on_comboBoxDevice_currentIndexChanged(const QString text)
{
   strcpy(usbDevice, text.toAscii());
   tell(eloAlways, "set device to '%s'", usbDevice);
}

void SetupDialog::on_comboBoxDevice_editTextChanged(const QString text)
{
   strcpy(usbDevice, text.toAscii());
   tell(eloAlways, "set device to '%s'", usbDevice);
}

//***************************************************************************
// On Course Name Editing Finished
//***************************************************************************

void SetupDialog::on_lineEditCourse_editingFinished()
{
   strcpy(courseName, lineEditCourse->text().toAscii());
}

//***************************************************************************
// On Database Name Editing Finished
//***************************************************************************

void SetupDialog::on_lineEditDbName_editingFinished()
{
   strcpy(databaseName, lineEditDbName->text().toAscii());
}

//***************************************************************************
// On Resource Path Editing Finished
//***************************************************************************

void SetupDialog::on_lineEditResourcePath_editingFinished()
{
   strcpy(resourcePath, lineEditResourcePath->text().toAscii());
}

//***************************************************************************
// On Lap Race Editing Finished
//***************************************************************************

void SetupDialog::on_timeEditLapRace_editingFinished()
{
   maxTimeLapRace = 
      timeEditLapRace->time().hour()*3600 
      + timeEditLapRace->time().minute()*60 
      + timeEditLapRace->time().second();
}

//***************************************************************************
// On Training Editing Finished
//***************************************************************************

void SetupDialog::on_timeEditTraining_editingFinished()
{
   maxTimeTraining = 
      timeEditTraining->time().hour()*3600 
      + timeEditTraining->time().minute()*60 
      + timeEditTraining->time().second();
}

//***************************************************************************
// On Tab Widget Changed
//***************************************************************************

void SetupDialog::on_tabWidget_currentChanged(int tab)
{
   if (tab == 1)
      lineEditDriver->setFocus();
}

//***************************************************************************
// Speed Factor
//***************************************************************************

void SetupDialog::on_radioButtonFaktor1_toggled(bool checked)
{
   if (checked) speedFactor = 1;
}

void SetupDialog::on_radioButtonFaktor24_toggled(bool checked)
{
   if (checked) speedFactor = 24;
}

void SetupDialog::on_radioButtonFaktor32_toggled(bool checked)
{
   if (checked) speedFactor = 32;
}

void SetupDialog::on_radioButtonFaktor43_toggled(bool checked)
{
   if (checked) speedFactor = 43;
}

//***************************************************************************
// Jump The Gun
//***************************************************************************

void SetupDialog::on_radioButtonJumpTheGunAbort_toggled(bool checked)
{
   if (checked) abortAtJumpTheGun = true;
}

//***************************************************************************
// 
//***************************************************************************

void SetupDialog::on_radioButtonJumpTheGunPenalty_toggled(bool checked)
{
   if (checked) abortAtJumpTheGun = false;
}

void SetupDialog::on_spinBoxJumpTheGunPenaltyTime_valueChanged(int value)
{
   penaltyAtJumpTheGun = value;
}

void SetupDialog::on_doubleSpinBoxFuelMax_valueChanged(double value)
{
   fuelMax = value;
}

void SetupDialog::on_doubleSpinBoxFuelPerLap_valueChanged(double value)
{
   fuelPerLap = value;
}

void SetupDialog::on_spinBoxFuelingDelay_valueChanged(int value)
{
   fuelingDelay = value;
}

void SetupDialog::on_spinBoxFuelingPerSecond_valueChanged(int value)
{
   fuelingPerSecond = value;
}

void SetupDialog::on_spinBoxFuelFactorFastLap_valueChanged(int value)
{
   fuelFactorFastLap = value;
}

void SetupDialog::on_spinBoxFuelFactorSlowLap_valueChanged(int value)
{
   fuelFactorSlowLap = value;
}

void SetupDialog::on_doubleSpinBoxAverageLap_valueChanged(double value)
{
   averageLap = value;
}

void SetupDialog::on_checkBoxFueling_stateChanged(int state)
{
   fuelingActive = state;
   groupBoxFueling->setEnabled(state);
}

void SetupDialog::on_spinBoxFuelPenaltyTime_valueChanged(int value)
{
   fuelPenaltyTime = value;
}

void SetupDialog::inputSignalChanged(int row, int col)
{
   QString value = tableWidgetInputSignals->item(row, col)->data(Qt::DisplayRole).toString();

   if (col == 1)
   {
      if (value == "NA")
         inputBits[row].bit = na;
      else
         inputBits[row].bit = value.toInt();
   }
   else if (col == 2)
   {
      if (value == "rising")
         inputBits[row].mode = teRising;
      else if (value == "falling")
         inputBits[row].mode = teFalling;
   }
}

void SetupDialog::outputSignalChanged(int row, int col)
{
   QString value = tableWidgetOutputSignals->item(row, col)->data(Qt::DisplayRole).toString();

   if (col == 1)
   {
      if (value == "NA")
         outputBits[row].bit = na;
      else
         outputBits[row].bit = value.toInt();
   }
   else if (col == 2)
   {
      if (value == "normal")
         outputBits[row].mode = omNormal;
      else if (value == "pwm")
         outputBits[row].mode = omPwm;
   }
}

//***************************************************************************
// 
//***************************************************************************

byte SetupDialog::getInputMask()
{
   byte mask = 0;

   for (int i = 0; i < bitInputCount; ++i) 
   {
      if (inputBits[i].bit < 8)
         mask |= 1 << inputBits[i].bit;
   }

   char buf[50];
   tell(eloDebug, "input Mask is '%s'", toBinStr(mask, buf, 8));

   return mask;
}

byte SetupDialog::getOutputMask()
{
   byte mask = 0;

   for (int i = 0; i < bitOutputCount; ++i) 
   {
      if (outputBits[i].bit < 8)
         mask |= 1 << outputBits[i].bit;
   }

   char buf[50];
   tell(eloDebug, "output Mask is '%s'", toBinStr(mask, buf, 8));

   return mask;
}
